#!/usr/bin/python

import sys
import math

def lattice(orig, reord, origweights, reordweights):
  # escape quotes
  orig = orig.replace("'", "\\'")
  reord = reord.replace("'", "\\'")

  # split sentences into words
  origwords = orig.split()
  reordwords = reord.split()
  origlen = len(origwords)
  reordlen = len(reordwords)

  # work out which sentence is longer
  if origlen <= reordlen:
    smallwords = origwords
    smalllen = origlen
    smallweights = origweights
    longwords = reordwords
    longlen = reordlen
    longweights = reordweights
  else:
    smallwords = reordwords
    smalllen = reordlen
    smallweights = reordweights
    longwords = origwords
    longlen = origlen
    longweights = origweights

  # weights to put on the edges of the non-branching nodes
  otherweights = len(origweights)*[1.0]

  # branching node
  lattice = "((('" + smallwords[0] + "'," + ",".join(map(str, smallweights)) + ",1),"
  lattice += "('" + longwords[0] + "'," + ",".join(map(str, longweights)) + ",2),),"

  # alternate nodes from shorter and longer sentence
  for i in xrange(1, smalllen-1):
    lattice += "(('" + smallwords[i] + "'," + ",".join(map(str, otherweights)) + ",2),),"
    lattice += "(('" + longwords[i] + "'," + ",".join(map(str, otherweights)) + ",2),),"

  # edge from last node in shorter sentence to final node
  lattice += "(('" + smallwords[smalllen-1] + "'," + ",".join(map(str, otherweights)) + "," + str(longlen-smalllen+2) + "),),"

  # remaining nodes for the longer sentence - there shouldn't be too many
  for j in xrange(smalllen-1, longlen):
    lattice += "(('" + longwords[j] + "'," + ",".join(map(str, otherweights)) + ",1),),"

  lattice += ")"

  return lattice


if __name__ == "__main__":
  if len(sys.argv) not in [ 4, 5 ]:
    sys.exit("Usage is " + sys.argv[0] + " original-file reordered-file output-file (features-file)")

  origfile = open(sys.argv[1])
  reordfile = open(sys.argv[2])
  outfile = open(sys.argv[3], 'w')

  if len(sys.argv) == 5:
    usefeatures = True
    featsfile = open(sys.argv[4])

    # set up check that every sentence has the same feature set
    firstline = True
    firstfeatures = {}
  else:
    usefeatures = False

  for (orig, reord) in zip(origfile, reordfile):
    reord_features = [1.0]

    if usefeatures:
      features = eval(featsfile.readline())

      # ensure we add the values to reord_features in a consistent order
      sorted = features.items()
      sorted.sort()

      # check for same feature set - clone the first set; check later set has same number of features
      if firstline:
        firstfeatures = features.copy()
      else:
        assert len(features) == len(firstfeatures)

      for (name, feature) in sorted:
        # check for same feature set - compare name and size of each feature to first set
        if not firstline:
          assert name in firstfeatures
          if type(feature) == list:
            assert type(firstfeatures[name]) == list
            assert len(feature) == len(firstfeatures[name])
          else:
            assert type(firstfeatures[name]) != list

        # features may be a single number or a list of numbers
        if type(feature) == list:
          reord_features.extend(feature)
        else:
          reord_features.append(feature)

      firstline = False
      
    orig_features = [2.718] + (len(reord_features)-1)*[1.0]
    outfile.write(lattice(orig, reord, orig_features, reord_features) + '\n')
  
  origfile.close()
  reordfile.close()
  outfile.close()

  if usefeatures:
    featsfile.close()
